//-----------------------------------------------------------------------------------
// A simple code that test the MPS algorighm.
//-----------------------------------------------------------------------------------

#include "mps.h"
#include <iostream>
#include <cstdlib>
#include <climits>
#include <string>
#include <algorithm>
#include <iterator>
#include <random>
#include <vector>
 
#include <ogdf/fileformats/GraphIO.h>
#define START_TEMP 100
// #define TIME

using namespace std;

int compute_removed_edge_size(const ogdf::Graph &G, vector<int> post_order);

// these functions are defined in mps_test.cpp
// but their signatures are not in mps.h, hence they are declared here
ogdf::Graph read_from_gml(string input_file);
vector<int> generate_post_order(const ogdf::Graph &G);
vector<int> generate_mutated_post_order(const ogdf::Graph &G, const vector<int> &post_order, int mutate_start_point);
vector<int> generate_guided_post_order(const ogdf::Graph &G, const vector<int> &post_order);
void compute_mps(const ogdf::Graph &G, int mutate_point, vector<int> &post_order, int &return_edge_size);

void vector_printer(const vector<int>& state) {
    for (size_t i = 0; i < state.size(); ++i) {
        std::cout << state[i] << ",";
    }
    std::cout << std::endl;
}


vector<int> repeated_mutation(const ogdf::Graph &G, int k_max, trie* trie_store) {
    // generate first post order
    // std::cout << "generate first post order" << std::endl;
    vector<int> old_order = generate_post_order(G);
    vector_printer(old_order);
    vector<int> temp_order = old_order;
    int new_removed_size;
    int old_removed_size = INT_MAX;


    for (int k = 0; k < k_max; ++k) {

        // compute_mps(G, dist(gen), temp_order, new_removed_size);
        // we set mutate_start_point to 0 to mutate everything every round
        temp_order = generate_mutated_post_order(G, temp_order, 1);
        // temp_order = generate_guided_post_order(G, temp_order);
        temp_order = generate_guided_post_order(G, temp_order);
        // test if something is in the trie_store
        // returns false when no order is found in trie_store
        if (!(trie_store->check_post_order(temp_order, 0))) {
            trie_store->add_new_post_order(temp_order, 0);
        } else {
            std::cout << "repeated mutation found" << std::endl;
        }
        new_removed_size = compute_removed_edge_size(G, temp_order);


        // if there is an improvement
        // 1. update the removed size to use the new smaller size
        // 2. update the old_order to be the new_order
        if (new_removed_size < old_removed_size) {
            old_removed_size = new_removed_size;
            old_order = temp_order;
        // if there is no improvement, we revert the temp_order to the old_order
        } else {
            temp_order = old_order;
        }
    }
    return old_order;
}


//-----------------------------------------------------------------------------------
// Main function.
//-----------------------------------------------------------------------------------


int main(int argc, char* argv[]) {
    string input_file = argv[1];
    int k_max = std::stoi(argv[2]);

    const ogdf::Graph G = read_from_gml(input_file);
    trie* trie_store = new trie(-1);

    // generate order here
    vector<int> post_order = repeated_mutation(G, k_max, trie_store);

    // test timing of function
    // test_correctness(G);   

    // // print final order and number of edges
    std::cout << "---" << std::endl;
    std::cout << "final report" << std::endl;
    std::copy(post_order.begin(), post_order.end(), std::ostream_iterator<int>(std::cout, ","));
    std::cout << std::endl;
    int removed_edges = compute_removed_edge_size(G, post_order);
    std::cout << "Number of removed edges: " << removed_edges << std::endl;

	return 0;
}
