/* The trie data structure is used to store all DFS traversals of the tree.
 * Although not strictly correct, we treat a canonical rotation representation as
 * a kind of traversal order. 
 * 
 * We will store a separate traversal tree for each starting root node. 
 * 
 * The objective is so that we can identify previously tried post-orders and not
 * blindly test a post-order that we already know the result to
 */
#include "mps.h"

//-----------------------------------------------------------------------------------
// CONSTRUCTOR
//-----------------------------------------------------------------------------------
trie::trie(int node_id) {
    _node_id = node_id;
}

//-----------------------------------------------------------------------------------
// METHODS
//-----------------------------------------------------------------------------------

// add post order to trie structure
// root Trie node will have current_position = 0
void trie::add_new_post_order(vector<int> post_order, int current_position) {
    int last_index = post_order.size() - 1;
    int current_index = last_index - current_position;
    // we only add when the current_index is still valid
    if (current_index >= 0) {
        int node_id = post_order[current_index];

        // only if it doesn't exist yet
        // we want to check for existing node_id in the _next_node_list
        // Assuming 'it' is an iterator pointing to an element in 'vec'
        auto it = std::find_if(_next_node_list.begin(), _next_node_list.end(), [node_id](trie* obj) {
            return obj->_node_id == node_id;
        });
        // only add if not found
        if (it == _next_node_list.end()) {
            // std::cout << "added trie node for: " << node_id << std::endl;
            trie* new_trie_node = new trie(node_id);
            new_trie_node->add_new_post_order(post_order, current_position+1);
            _next_node_list.push_back(new_trie_node);
        // if found, then just continue
        } else {
            (*it)->add_new_post_order(post_order, current_position+1);
        }
    }
}

bool trie::check_post_order(vector<int> post_order, int current_position) {
    bool return_value = false;
    int last_index = post_order.size() - 1;
    int current_index = last_index - current_position;
    // while current_index is still valid
    if (current_index >= 0) {
        int node_id = post_order[current_index];
        // we want to check for existing node_id in the _next_node_list
        // Assuming 'it' is an iterator pointing to an element in 'vec'
        auto it = std::find_if(_next_node_list.begin(), _next_node_list.end(), [node_id](trie* obj) {
            return obj->_node_id == node_id;
        });
        // found
        if (it != _next_node_list.end()) {
            // std::cout << (*it)->_node_id << std::endl;
            return_value = (*it)->check_post_order(post_order, current_position+1);
        // not found
        } else {
            ; // return_value is already false by default
        }
    } else { // we have reached the base case of reaching the end of the trie
        return true;
    }
    return return_value;
}